/*****************************************************************************************************************************
--
--    File Name    : RGB2YCbCr_tb.v 

--    Description  : This module provides the test environment for RGB to YCbCr IP
--                   
					 The following file is required for the simulation :
					 RGB_in.txt -->  It contains hex values which are in the following format: 
									 First 2 bytes represent width of the image, next 2 bytes height of the image.
									 Then follows B,G and R values.
									 For more details visit Microsemi Video Web Page.

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1fs
module RGB2YCbCr_test;
  /************************************************************************
          Parameters 
  *************************************************************************/
  parameter HEIGHT = 200;
  parameter WIDTH = 320;
  parameter CLKPERIOD = 10;
  parameter DATA_BIT_WIDTH = 24;
  parameter IMAGE_FILE_NAME = "RGB_in.txt";  
  //Parameters from the DUT
  parameter g_RGB_DATA_BIT_WIDTH = 8;
  parameter g_YCbCr_DATA_BIT_WIDTH = 8;
  localparam MAX_SIZE = (HEIGHT * WIDTH * (DATA_BIT_WIDTH / g_RGB_DATA_BIT_WIDTH)) + 4;
  localparam MAX_SIZE_24 = (HEIGHT * WIDTH) + 1;
  /************************************************************************
          Inputs and Outputs to the DUT 
  *************************************************************************/
  reg Clock;
  reg Reset;
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Red;
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Green;
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] Blue;
  reg dataValidIn;
  reg H_sync_in;
  reg V_sync_in;
  reg H_active_in;
  reg V_active_in;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Y_out;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cb_out;
  wire[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Cr_out;
  wire dataValidOut;
  wire H_sync_out;
  wire V_sync_out;
  wire H_active_out;
  wire V_active_out;
  reg[31 : 0] counter_h,
              counter_v,
              counter_h_in,
              counter_v_in;
  reg[1 : 0] counter_line_wait;
  reg[3 : 0] count_done;
  reg[1 : 0] state;
  localparam[1 : 0] IDLE = 0,
                    READ = 1,
                    WAIT = 2,
                    DONE = 3;
  reg[g_RGB_DATA_BIT_WIDTH - 1 : 0] image_8[MAX_SIZE - 1 : 0];
  reg[DATA_BIT_WIDTH - 1 : 0] image_24[MAX_SIZE_24 - 1 : 0];
  integer i,
          j;
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_Cr[MAX_SIZE_24 - 2 : 0];
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_Cb[MAX_SIZE_24 - 2 : 0];
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_Y[MAX_SIZE_24 - 2 : 0];
  reg[g_YCbCr_DATA_BIT_WIDTH - 1 : 0] Mem_final[MAX_SIZE - 5 : 0];
  integer k,
          l,
          m,
	  File_1;
  initial
    begin
      Reset = 0;
      Clock = 0;
      dataValidIn = 0;
      H_sync_in = 0;
      V_sync_in = 0;
      H_active_in = 0;
      V_active_in = 0;
      Red = 0;
      Green = 0;
      Blue = 0;
      j = 1;
      k = 0;
      l = 0;
      m = 0;
      counter_v_in = 0;
      counter_h_in = 0;
      counter_v = 0;
      counter_h = 0;
      m = 0;
      l = 0;
      counter_line_wait = 0;
      count_done = 0;
    end
  /************************************************************************
          Generates Clock signal
  *************************************************************************/
  initial
    begin
      forever
        #(CLKPERIOD / 2)
        Clock = ~Clock;
    end
  /************************************************************************
          RESET : Generates a Reset pulse 
  *************************************************************************/
  initial
    begin
    File_1 = $fopen("YCbCr_out.txt","wb");
      Reset = 1;
      #4
      ;
      Reset = 0;
      #2
      ;
      Reset = 1;
    end
  /************************************************************************
          Read the input image 
  *************************************************************************/
  initial
    begin
      $readmemh(IMAGE_FILE_NAME, image_8);
      for (i = 0; i < MAX_SIZE_24; i = i + 1)
        begin
          image_24[i] = (({ 16'b0, image_8[i * 3 + 1] } << 16) | ({ 16'b0, image_8[i * 3 + 2] } << 8) | { image_8[i * 3 + 3] });
        end
      $display("");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
      $display(">>>>>>>>                     IMAGE LOADED                                           <<<<<<<");
      $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
    end
  /************************************************************************
          Debug Signals: Output horizontal counter and output vertical counter
  *************************************************************************/
  always
    @(posedge Clock)
    begin
      if (counter_h == (WIDTH))
        begin
          counter_h <= 0;
          counter_v <= counter_v + 1;
        end
      else
        if (dataValidOut)
          counter_h <= counter_h + 1;
    end
  /************************************************************************
          Debug Signals: Input vertical counter
  *************************************************************************/
  always
    @(posedge Clock)
    begin
      if ((Reset == 1) && (counter_line_wait == 2'd1) && (counter_v_in < HEIGHT))
        counter_v_in <= counter_v_in + 1;
      else
        begin
          counter_v_in <= counter_v_in;
        end
    end
  /******************************************************************************
          FSM to process the input image 
  *******************************************************************************/
  always
    @(posedge Clock)
    begin
      if (!Reset)
        begin
          state <= 0;
        end
      else
        begin
          case (state)
            IDLE:
              begin
                $display("");
                $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                $display(">>>>>>>>                    PROCESSING OF IMAGE STARTED                             <<<<<<<");
                $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                state <= READ;
                H_sync_in <= 1;
                V_sync_in <= 1;
                H_active_in <= 1;
                V_active_in <= 1;
              end
            READ:
              begin
                if ((counter_h_in == (WIDTH)) && (Reset == 1))
                  begin
                    dataValidIn <= 0;
                    state <= WAIT;
                    H_sync_in <= 0;
                    V_sync_in <= 0;
                    H_active_in <= 0;
                    V_active_in <= 0;
                  end
                else
                  begin
                    if ((counter_h_in < (WIDTH)) && (Reset == 1))
                      begin
                        dataValidIn <= 1;
                        H_sync_in <= 1;
                        V_sync_in <= 1;
                        H_active_in <= 1;
                        V_active_in <= 1;
                        Red <= image_24[j][7 : 0];
                        Green <= image_24[j][15 : 8];
                        Blue <= image_24[j][23 : 16];
                        j <= j + 1;
                        counter_h_in <= counter_h_in + 1;
                        state <= READ;
                      end
                    else
                      state <= READ;
                  end
              end
            WAIT:
              begin
                if (counter_v_in == HEIGHT)
                  begin
                    dataValidIn <= 0;
                    state <= DONE;
                    H_sync_in <= 0;
                    V_sync_in <= 0;
                    H_active_in <= 0;
                    V_active_in <= 0;
                  end
                else
                  begin
                    if (counter_line_wait == 2'd3)
                      begin
                        counter_line_wait <= 0;
                        counter_h_in <= 0;
                        state <= READ;
                        H_sync_in <= 0;
                        V_sync_in <= 0;
                        H_active_in <= 0;
                        V_active_in <= 0;
                      end
                    else
                      begin
                        counter_line_wait <= counter_line_wait + 1;
                        state <= WAIT;
                        H_sync_in <= 0;
                        V_sync_in <= 0;
                        H_active_in <= 0;
                        V_active_in <= 0;
                      end
                  end
              end
            DONE:
              begin
                if (count_done == 4'd15)
                  begin
                    $display("");
                    $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                    $display(">>>>>>>>        \t\t\tSIMULATION COMPLETED\t    \t\t\t\t\t\t      <<<<<<<");
                    $display("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");
                    $finish;
                  end
                else
                  begin
                    if (counter_v == HEIGHT)
                      begin
                        count_done <= count_done + 1;
                        state <= DONE;
                      end
                  end
              end
          endcase
        end
    end
  /************************************************************************
          Process to write the output YCbCr into a memory
  *************************************************************************/
  always
    @(posedge Clock)
    begin
      if (dataValidOut)
        begin
	  $fwrite(File_1,"%x\n",Cr_out);
	  $fwrite(File_1,"%x\n",Cb_out);
	  $fwrite(File_1,"%x\n",Y_out);
          k <= k + 1;
        end
    end

  /************************************************************************
          Component instantiation
  *************************************************************************/
  RGB2YCbCr #(.g_RGB_DATA_BIT_WIDTH(g_RGB_DATA_BIT_WIDTH), .g_YCbCr_DATA_BIT_WIDTH(g_YCbCr_DATA_BIT_WIDTH)) R2Y_Converter_0(.CLOCK_I(Clock),
                                                                                                                            .RESET_N_I(Reset),
                                                                                                                            .Red_i(Red),
                                                                                                                            .Green_i(Green),
                                                                                                                            .Blue_i(Blue),
                                                                                                                            .data_enable_i(dataValidIn),
                                                                                                                            .horz_sync_i(H_sync_in),
                                                                                                                            .vert_sync_i(V_sync_in),
                                                                                                                            .hactive_i(H_active_in),
                                                                                                                            .vactive_i(V_active_in),
                                                                                                                            .Y_out_o(Y_out),
                                                                                                                            .Cb_out_o(Cb_out),
                                                                                                                            .Cr_out_o(Cr_out),
                                                                                                                            .data_enable_o(dataValidOut),
                                                                                                                            .horz_sync_o(H_sync_out),
                                                                                                                            .vert_sync_o(V_sync_out),
                                                                                                                            .hactive_o(H_active_out),
                                                                                                                            .vactive_o(V_active_out));
endmodule
