/*****************************************************************************************************************************
--
--    File Name    : data_unpack_64_8.v 

--    Description  : This module implements 64-bit to 8-bit unpacker for read channels 2, 3 and 4
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module unpack_64_8 #(parameter g_AXI_DWIDTH = 64,
                     parameter g_AXI_BUFF_AWIDTH = 10,
                     parameter g_RD_CHANNEL_VIDEO_DATA_WIDTH = 8)
                   (input wire clock,
                    input wire nreset,
                    input wire start_unpack,
                    input wire[(g_AXI_BUFF_AWIDTH - 1) : 0] locs_to_read,
                    input wire[(g_AXI_DWIDTH - 1) : 0] buff_data_in,
                    output reg[(g_AXI_BUFF_AWIDTH - 1) : 0] buff_raddr,
                    output reg[(g_RD_CHANNEL_VIDEO_DATA_WIDTH - 1) : 0] video_data,
                    output reg data_valid,
                    output reg unpacker_compl);
  parameter[3 : 0] lol = 4'd0,
                   Oil = 4'd1,
                   Iil = 4'd2,
                   lil = 4'd3,
                   oil = 4'd4,
                   iil = 4'd5,
                   OO0 = 4'd6,
                   IO0 = 4'd7,
                   lO0 = 4'd8;
  reg[3 : 0] oO0;
  reg[(g_AXI_DWIDTH - 1) : 0] iO0;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          iO0 <= { g_AXI_DWIDTH { 1'b0 } };
        end
      else
        begin
          iO0 <= buff_data_in;
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        oO0 <= lol;
      else
        begin
          case (oO0)
            lol:
              begin
                if (start_unpack)
                  oO0 <= Oil;
                else
                  oO0 <= lol;
              end
            Oil:
              begin
                oO0 <= Iil;
              end
            Iil:
              begin
                oO0 <= lil;
              end
            lil:
              begin
                oO0 <= oil;
              end
            oil:
              begin
                oO0 <= iil;
              end
            iil:
              begin
                oO0 <= OO0;
              end
            OO0:
              begin
                oO0 <= IO0;
              end
            IO0:
              begin
                oO0 <= lO0;
              end
            lO0:
              begin
                if (buff_raddr == locs_to_read)
                  oO0 <= lol;
                else
                  oO0 <= Oil;
              end
            default:
              oO0 <= lol;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
          buff_raddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          unpacker_compl <= 1'b0;
          data_valid <= 1'b0;
        end
      else
        begin
          case (oO0)
            lol:
              begin
                video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
                buff_raddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                unpacker_compl <= 1'b0;
                data_valid <= 1'b0;
              end
            Oil:
              begin
                video_data <= iO0[7 : 0];
                data_valid <= 1'b1;
              end
            Iil:
              begin
                video_data <= iO0[15 : 8];
                data_valid <= 1'b1;
              end
            lil:
              begin
                video_data <= iO0[23 : 16];
                data_valid <= 1'b1;
              end
            oil:
              begin
                video_data <= iO0[31 : 24];
                data_valid <= 1'b1;
              end
            iil:
              begin
                video_data <= iO0[39 : 32];
                data_valid <= 1'b1;
              end
            OO0:
              begin
                video_data <= iO0[47 : 40];
                data_valid <= 1'b1;
                buff_raddr <= buff_raddr + 1'b1;
              end
            IO0:
              begin
                video_data <= iO0[55 : 48];
                data_valid <= 1'b1;
              end
            lO0:
              begin
                video_data <= iO0[63 : 56];
                data_valid <= 1'b1;
                if (buff_raddr == locs_to_read)
                  unpacker_compl <= 1'b1;
                else
                  unpacker_compl <= 1'b0;
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
