/*****************************************************************************************************************************
--
--    File Name    : WriteChannel_2_tb.v 

--    Description  : DDR Arbiter Write Channel 2 Test Bench
					 

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1 ns / 1 ps
module write2 
 #(
   parameter g_DATA_WIDTH = 24,
   parameter g_DDR_ADDRESS_WIDTH = 32,
   parameter g_WR_CHANNEL_AXI_BUFF_AWIDTH = 13,
   parameter g_RD_CHANNEL_AXI_BUFF_AWIDTH = 13
 )(
   input wire clock_i,
   input wire nreset_i,
   input wire tb_start_i,
   output wire [(g_DDR_ADDRESS_WIDTH-1):0] start_write_ddr_address_o,
   output reg [(g_WR_CHANNEL_AXI_BUFF_AWIDTH + 3 - 1):0]bytes_to_write_o,
   output reg [(g_DATA_WIDTH-1):0] write_data_o,
   output reg write_data_valid_o,
   output reg write_req_o,
   input wire write_ack_i,
   input wire write_done_i,
   output reg [31:0] counter_v_rgb
 );

parameter IMAGE_2_FILE_NAME = "RGB_in.txt";

parameter DDR_START_ADDRESS = 32'h8D000000;
parameter HEIGHT=200;
parameter WIDTH=320;
parameter TOTAL_BYTES_TO_TRANSFER = WIDTH*3;
parameter MAX_SIZE=(HEIGHT*WIDTH*3)+4;
parameter MAX_SIZE_32=(HEIGHT*WIDTH)+1;


 reg [(g_WR_CHANNEL_AXI_BUFF_AWIDTH + 3 - 1):0] bytes_counter, data_counter;
 reg data_counter_en, data_gen_done;
 reg [(g_DDR_ADDRESS_WIDTH-1):0] start_write_ddr_address_reg;

 assign start_write_ddr_address_o = start_write_ddr_address_reg;

 parameter [3:0] beat_gen_tb_idle = 4'd0,
                 write_data_gen_state = 4'd1,
                 gen_write_trans = 4'd2, 
                 wait_for_write_ack = 4'd3,
                 wait_for_write_done = 4'd4, 
                 check_num_of_bytes = 4'd5,
                 tb_compl = 4'd6;
  reg [3:0] beat_gen_tb_fsm;


reg [7:0] image_8[MAX_SIZE-1:0];
reg [31:0] image_32[MAX_SIZE_32-1:0];
integer i,j,counter_h_rgb;



initial 
begin
  $readmemh(IMAGE_2_FILE_NAME, image_8);
 for (i = 0; i < MAX_SIZE_32; i = i + 1) begin
   	 image_32[i] = ( ({16'b0, image_8[i*3 +1 ]} <<16) | ({16'b0, image_8[i*3 + 2]} << 8) | { image_8[i*3 + 3]}  );
  end
end


 always @(posedge clock_i or negedge nreset_i)
 begin
  if (!nreset_i)
  begin
   beat_gen_tb_fsm <= beat_gen_tb_idle;
   bytes_counter <= TOTAL_BYTES_TO_TRANSFER;
   data_counter_en <= 1'b0;
   start_write_ddr_address_reg <= { g_DDR_ADDRESS_WIDTH { 1'b0 } };
   bytes_to_write_o <= {(g_WR_CHANNEL_AXI_BUFF_AWIDTH + 3) {1'b0}};
   write_req_o <= 1'b0;
   counter_v_rgb <= 0;
    j <= 1;
   counter_h_rgb <= 0;
  end
  else
  begin
    case (beat_gen_tb_fsm)
     beat_gen_tb_idle       :begin
                               if (tb_start_i == 1'b1)
                               begin
                                start_write_ddr_address_reg <= DDR_START_ADDRESS;
                                bytes_counter <= TOTAL_BYTES_TO_TRANSFER;
                                beat_gen_tb_fsm <= write_data_gen_state;
                               end
                               else
                               begin
                                beat_gen_tb_fsm <= beat_gen_tb_idle;
                                bytes_counter <= 'd0;
                               end                         
                             end
     write_data_gen_state   :begin                              
                              if (data_gen_done)
                              begin
                               beat_gen_tb_fsm <= gen_write_trans;
                               data_counter_en <= 1'b0;
                                
                              end
                              else
                              begin
                               data_counter_en <= 1'b1;
                               beat_gen_tb_fsm <= write_data_gen_state;
                              end
                             end                                 
     gen_write_trans        :begin
                              start_write_ddr_address_reg <= start_write_ddr_address_reg;
                              bytes_to_write_o <= bytes_counter;
                              write_req_o <= 1'b1;
                              beat_gen_tb_fsm <= wait_for_write_ack;
                             end
     wait_for_write_ack     :begin
                              if (write_ack_i)
                              begin
                               write_req_o <= 1'b0;
                               beat_gen_tb_fsm <= wait_for_write_done;
                              end
                              else
                              begin
                               write_req_o <= 1'b1;
                               beat_gen_tb_fsm <= wait_for_write_ack;
                              end
                             end
     wait_for_write_done    :begin
                              if (write_done_i)
                              begin                              
                               beat_gen_tb_fsm <= check_num_of_bytes;
                                counter_v_rgb <= counter_v_rgb + 1;
                              end
                              else
                              begin                               
                               beat_gen_tb_fsm <= wait_for_write_done;
                              end
                             end                             
                          
      check_num_of_bytes    :begin
                              if (counter_v_rgb == HEIGHT)
                              begin
                               start_write_ddr_address_reg <= DDR_START_ADDRESS;
                               beat_gen_tb_fsm <= tb_compl;
                               $display("WRITE CHANNEL 2: FRAME WRITE COMPLETE");
                              end
                              else
                              begin
                                start_write_ddr_address_reg <= start_write_ddr_address_reg + TOTAL_BYTES_TO_TRANSFER;
                                beat_gen_tb_fsm <= write_data_gen_state;
                              end
                             end
     tb_compl               :beat_gen_tb_fsm <= tb_compl;
                             
    endcase
  end
 end

 always @(posedge clock_i or negedge nreset_i)
 begin
  if (!nreset_i)
  begin
   write_data_o <= {g_DATA_WIDTH {1'b0}};
   write_data_valid_o <= 1'b0;
   data_counter <= 10'd0;   
   data_gen_done <= 1'b0;
  end
  else
  begin
   if (data_counter_en)
   begin
    if (data_counter == (bytes_counter/(g_DATA_WIDTH/8)) || (counter_h_rgb == (WIDTH)))
    begin
     write_data_o <= write_data_o;
     write_data_valid_o <= 1'b0;
     data_counter <= data_counter;
     data_gen_done <= 1'b1;
     counter_h_rgb <= 0; 
    end
    else
    begin
     write_data_o <=  image_32[j][31:0];
     j <= j+1;
     counter_h_rgb <= counter_h_rgb +1;
     write_data_valid_o <= 1'b1;
     data_counter <= data_counter + 1'b1; 
     data_gen_done <= 0;
    end
   end
   else
   begin
    write_data_o <= write_data_o;
    write_data_valid_o <= 1'b0;
    data_counter <= 10'd0;
    data_gen_done <= 1'b0;
   end
  end
 end

 endmodule
