/*****************************************************************************************************************************
--
--    File Name    : video_timing_generator.v

--    Description  : This module generates timing for a specified display resolution
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1ns/1ps
module video_timing_generator #(parameter g_INPUT_X_W_RES_WIDTH = 12,
                                parameter g_INPUT_Y_H_RES_WIDTH = 12,
                                parameter g_INPUT_VIDEO_DATA_BIT_WIDTH = 24,
                                parameter g_HORZ_SYNC_PULSE_POLARITY = 1,
                                parameter g_VERT_SYNC_PULSE_POLARITY = 1)
                              (input wire RESET_N_I,
                               input wire VIDEO_CLOCK_I,
                               input wire display_control_enable_i,
                               input wire[(g_INPUT_X_W_RES_WIDTH - 1) : 0] tot_horz_pixels_i,
                               input wire[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] tot_vert_lines_i,
                               input wire[31 : 0] video_resolution_i,
                               input wire[31 : 0] back_porch_width_i,
                               input wire[31 : 0] front_porch_width_i,
                               input wire[(g_INPUT_X_W_RES_WIDTH - 1) : 0] hsync_width_i,
                               input wire[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] vsync_width_i,
                               output reg disp_cont_busy_o,
                               output reg disp_cont_done_o,
                               output reg start_ddr_read,
                               input wire[(g_INPUT_VIDEO_DATA_BIT_WIDTH - 1) : 0] video_data_in,
                               output wire fifo_re,
                               output wire[23 : 0] rgb_out_o,
                               output wire data_enable_o,
                               output wire horz_sync_o,
                               output wire vert_sync_o,
                               output wire hactive_o,
                               output wire vactive_o);
  reg lil,
      oil;
  wire iil;
  reg[(g_INPUT_X_W_RES_WIDTH - 1) : 0] OO0;
  reg[(g_INPUT_X_W_RES_WIDTH - 1) : 0] IO0;
  reg[(g_INPUT_X_W_RES_WIDTH - 1) : 0] lO0;
  reg[(g_INPUT_X_W_RES_WIDTH - 1) : 0] oO0;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] iO0;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] OI0;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] II0;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] lI0;
  reg[(g_INPUT_X_W_RES_WIDTH - 1) : 0] oI0;
  reg[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] iI0;
  reg Ol0;
  reg Il0;
  wire ll0;
  wire ol0;
  reg[(g_INPUT_VIDEO_DATA_BIT_WIDTH - 1) : 0] il0;
  reg ii,
      OOI;
  reg O00;
  parameter[2 : 0] i1I = 3'd0,
                   OoI = 3'd1,
                   I00 = 3'd2,
                   l00 = 3'd3;
  reg[2 : 0] o00;
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          ii <= 1'b0;
          OOI <= 1'b0;
        end
      else
        begin
          ii <= display_control_enable_i;
          OOI <= ii;
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          il0 <= { g_INPUT_VIDEO_DATA_BIT_WIDTH { 1'b0 } };
        end
      else
        begin
          if (ol0)
            il0 <= video_data_in;
        end
    end
  assign rgb_out_o = iil ? il0 : 24'd0;
  assign data_enable_o = iil ? 1'b1 : 1'b0;
  assign horz_sync_o = Ol0;
  assign fifo_re = ll0;
  assign hactive_o = lil;
  assign vactive_o = O00;
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          o00 <= i1I;
        end
      else
        begin
          case (o00)
            i1I:
              begin
                if (OOI)
                  o00 <= OoI;
                else
                  o00 <= i1I;
              end
            OoI:
              begin
                o00 <= I00;
              end
            I00:
              begin
                o00 <= l00;
              end
            l00:
              begin
                if ((iI0 == (iO0 - 1)) && (oI0 == (OO0 - 1)))
                  o00 <= I00;
                else
                  o00 <= l00;
              end
            default:
              o00 <= i1I;
          endcase
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          Il0 <= 1'b0;
          disp_cont_busy_o <= 1'b0;
          disp_cont_done_o <= 1'b0;
          OO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
          iO0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
          IO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
          OI0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
          lO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
          II0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
          oO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
          lI0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
          start_ddr_read <= 1'b0;
        end
      else
        begin
          case (o00)
            i1I:
              begin
                Il0 <= 1'b0;
                disp_cont_busy_o <= 1'b0;
                disp_cont_done_o <= 1'b0;
                OO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
                iO0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
                IO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
                OI0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
                lO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
                II0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
                oO0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
                lI0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
                start_ddr_read <= 1'b0;
              end
            OoI:
              begin
                OO0 <= tot_horz_pixels_i;
                iO0 <= tot_vert_lines_i;
                oO0 <= video_resolution_i[g_INPUT_X_W_RES_WIDTH - 1 : 0];
                lI0 <= video_resolution_i[27 : 16];
                lO0 <= back_porch_width_i[11 : 0];
                II0 <= back_porch_width_i[27 : 16];
                IO0 <= front_porch_width_i[11 : 0];
                OI0 <= front_porch_width_i[27 : 16];
              end
            I00:
              begin
                Il0 <= 1'b1;
                disp_cont_busy_o <= 1'b1;
                disp_cont_done_o <= 1'b0;
                start_ddr_read <= 1'b1;
              end
            l00:
              begin
                if ((iI0 == (iO0 - 1)) && (oI0 == (OO0 - 1)))
                  begin
                    disp_cont_busy_o <= 1'b0;
                    disp_cont_done_o <= 1'b1;
                    start_ddr_read <= 1'b0;
                    Il0 <= Il0;
                  end
                else
                  begin
                    disp_cont_busy_o <= disp_cont_busy_o;
                    Il0 <= Il0;
                    start_ddr_read <= start_ddr_read;
                    disp_cont_done_o <= 1'b0;
                  end
              end
            default:
              begin
              end
          endcase
        end
    end
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          oI0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
          Ol0 <= ~g_HORZ_SYNC_PULSE_POLARITY;
          lil <= 1'b0;
        end
      else
        if (Il0)
          begin
            if (oI0 == (OO0 - 1))
              begin
                oI0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
                Ol0 <= ~g_HORZ_SYNC_PULSE_POLARITY;
                lil <= 1'b0;
              end
            else
              if ((oI0 >= (lO0 - 1)) && (oI0 < (oO0 + lO0 - 1)))
                begin
                  lil <= 1'b1;
                  oI0 <= oI0 + 1'b1;
                  Ol0 <= ~g_HORZ_SYNC_PULSE_POLARITY;
                end
              else
                if ((oI0 >= (oO0 + lO0 + IO0 - 1)) && (oI0 < (OO0 - 1)))
                  begin
                    lil <= 1'b0;
                    oI0 <= oI0 + 1'b1;
                    Ol0 <= g_HORZ_SYNC_PULSE_POLARITY;
                  end
                else
                  begin
                    lil <= 1'b0;
                    oI0 <= oI0 + 1'b1;
                    Ol0 <= ~g_HORZ_SYNC_PULSE_POLARITY;
                  end
          end
        else
          begin
            oI0 <= { g_INPUT_X_W_RES_WIDTH { 1'b0 } };
            Ol0 <= ~g_HORZ_SYNC_PULSE_POLARITY;
            lil <= 1'b0;
          end
    end
  assign ll0 = ((oI0 >= (lO0 - 2)) && (oI0 < (oO0 + lO0 - 2)) && (oil == 1'b1)) ? 1'b1 : 1'b0;
  assign ol0 = ((oI0 >= (lO0 - 2)) && (oI0 < (oO0 + lO0 - 1)) && (oil == 1'b1)) ? 1'b1 : 1'b0;
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          iI0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if ((iI0 == (iO0 - 1)) && (oI0 == (OO0 - 1)))
            begin
              iI0 <= { g_INPUT_Y_H_RES_WIDTH { 1'b0 } };
            end
          else
            if (oI0 == (OO0 - 1))
              begin
                iI0 <= iI0 + 1'b1;
              end
        end
    end
  assign vert_sync_o = ((iI0 >= (lI0 + II0 + OI0 - 1)) && (iI0 < (iO0 - 1))) ? g_VERT_SYNC_PULSE_POLARITY : ~g_VERT_SYNC_PULSE_POLARITY;
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          oil <= 1'b0;
        end
      else
        begin
          if ((iI0 >= (II0 - 1)) && (iI0 < (lI0 + II0 - 1)))
            begin
              oil <= 1'b1;
            end
          else
            begin
              oil <= 1'b0;
            end
        end
    end
  assign iil = oil & lil;
  always
    @(posedge VIDEO_CLOCK_I,
      negedge RESET_N_I)
    begin
      if (!RESET_N_I)
        begin
          O00 <= 1'b0;
        end
      else
        begin
          if ((iI0 == (II0 - 2)) && (oI0 == (oO0 + lO0 - 1)))
            begin
              O00 <= 1'b1;
            end
          else
            if ((iI0 == (lI0 + II0 - 2)) && (oI0 == (oO0 + lO0 - 1)))
              begin
                O00 <= 1'b0;
              end
            else
              begin
                O00 <= O00;
              end
        end
    end
endmodule
