/*****************************************************************************************************************************
--
--    File Name    : byte2pixel_conversion.v 

--    Description  : This module implements conversion of byte data to pixel data.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
module byte2pixel_conversion #(parameter g_DATAWIDTH = 8,
                               parameter g_LANE_WIDTH = 4)
                             (input CAM_CLOCK_I,
                              input READ_CLOCK_I,
                              input RESET_n_I,
                              input RESET_READ_CLK_SYNC,
                              input wire[g_LANE_WIDTH * 8 - 1 : 0] byte_data_i,
                              input wire byte_en_i,
                              output reg[g_DATAWIDTH - 1 : 0] pixel_data_o,
                              output reg line_valid_o,
                              output wire line_start_o,
                              output wire line_end_o);
  parameter O = 2'b00,
            I = 2'b01,
            l = 2'b10,
            o = 2'b11;
  parameter i = 3'b000,
            OI = 3'b001,
            II = 3'b010,
            lI = 3'b011,
            oI = 3'b100,
            iI = 3'b101,
            Ol = 3'b110,
            Il = 3'b111;
  parameter ll = (g_DATAWIDTH == 8) ? 8 * g_LANE_WIDTH : 10 * g_LANE_WIDTH;
  reg ol;
  reg il;
  reg[g_LANE_WIDTH * g_DATAWIDTH - 1 : 0] O0,
                                          I0,
                                          l0;
  wire[g_LANE_WIDTH * g_DATAWIDTH - 1 : 0] o0;
  reg[g_LANE_WIDTH * 8 - 1 : 0] i0,
                                O1,
                                I1,
                                l1,
                                o1;
  reg i1,
      Oo,
      Io;
  reg[3 : 0] lo;
  reg[2 : 0] oo;
  reg[4 : 0] io;
  wire Oi,
       Ii;
  reg li,
      oi;
  reg[1 : 0] ii,
             OOI;
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    if (!RESET_n_I)
      begin
        i0 <= 0;
        Io <= 0;
      end
    else
      begin
        i0 <= byte_data_i;
        Io <= byte_en_i;
      end
  generate
    if (g_DATAWIDTH == 8)
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              O0 <= 0;
              i1 <= 0;
            end
          else
            begin
              O0 <= i0;
              i1 <= Io;
            end
      end
  endgenerate
  generate
    if ((g_DATAWIDTH == 10) && (g_LANE_WIDTH == 4))
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          begin
            if (!RESET_n_I)
              oo <= 0;
            else
              if (Io)
                oo <= (oo == 3'b100) ? 3'b000 : oo + 1;
              else
                oo <= 0;
          end
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              O0 <= 0;
              i1 <= 0;
            end
          else
            begin
              O1 <= i0;
              I1 <= O1;
              if (oo > 0)
                begin
                  case (oo)
                    OI:
                      O0 <= { O1[31 : 24], i0[7 : 6], O1[23 : 16], i0[5 : 4], O1[15 : 8], i0[3 : 2], O1[7 : 0], i0[1 : 0] };
                    II:
                      O0 <= { i0[7 : 0], i0[15 : 14], O1[31 : 24], i0[13 : 12], O1[23 : 16], i0[11 : 10], O1[15 : 8], i0[9 : 8] };
                    lI:
                      O0 <= { i0[15 : 8], i0[23 : 22], i0[7 : 0], i0[21 : 20], O1[31 : 24], i0[19 : 18], O1[23 : 16], i0[17 : 16] };
                    oI:
                      O0 <= { i0[23 : 16], i0[31 : 30], i0[15 : 8], i0[29 : 28], i0[7 : 0], i0[27 : 26], O1[31 : 24], i0[25 : 24] };
                    default:
                      O0 <= 0;
                  endcase
                  i1 <= (oo[0] | oo[1] | oo[2]);
                end
              else
                begin
                  i1 <= 0;
                  O0 <= 0;
                end
            end
      end
  endgenerate
  generate
    if ((g_DATAWIDTH == 10) && (g_LANE_WIDTH == 2))
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          begin
            if (!RESET_n_I)
              oo <= 0;
            else
              if (Io)
                oo <= (oo == 3'b101) ? 3'b001 : oo + 1;
              else
                oo <= 0;
          end
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              O0 <= 0;
              i1 <= 0;
            end
          else
            begin
              O1 <= i0;
              I1 <= O1;
              if (oo > 0)
                begin
                  case (oo)
                    II:
                      O0 <= { I1[15 : 8], i0[3 : 2], I1[7 : 0], i0[1 : 0] };
                    lI:
                      O0 <= { I1[15 : 8], O1[7 : 6], I1[7 : 0], O1[5 : 4] };
                    oI:
                      O0 <= { O1[7 : 0], i0[11 : 10], I1[15 : 8], i0[9 : 8] };
                    iI:
                      O0 <= { O1[7 : 0], O1[15 : 14], I1[15 : 8], O1[13 : 12] };
                    default:
                      O0 <= 0;
                  endcase
                  i1 <= (oo >= 3'b010);
                end
              else
                begin
                  i1 <= 0;
                  O0 <= 0;
                end
            end
      end
  endgenerate
  generate
    if ((g_DATAWIDTH == 10) && (g_LANE_WIDTH == 1))
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          begin
            if (!RESET_n_I)
              oo <= 0;
            else
              if (Io)
                oo <= (oo == 3'b111) ? 3'b011 : oo + 1;
              else
                oo <= 0;
          end
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              O0 <= 0;
              i1 <= 0;
            end
          else
            begin
              O1 <= i0;
              I1 <= O1;
              l1 <= I1;
              o1 <= l1;
              if (oo > 0)
                begin
                  case (oo)
                    oI:
                      O0 <= { o1[7 : 0], i0[1 : 0] };
                    iI:
                      O0 <= { o1[7 : 0], O1[3 : 2] };
                    Ol:
                      O0 <= { o1[7 : 0], I1[5 : 4] };
                    Il:
                      O0 <= { o1[7 : 0], l1[7 : 6] };
                    default:
                      O0 <= 0;
                  endcase
                  i1 <= (oo >= 3'b100);
                end
              else
                begin
                  i1 <= 0;
                  O0 <= 0;
                end
            end
      end
  endgenerate
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    if (!RESET_READ_CLK_SYNC)
      lo <= 0;
    else
      if (Oi)
        lo <= 0;
      else
        if (lo <= 8)
          lo <= lo + 1;
        else
          lo <= lo;
  generate
    if (g_LANE_WIDTH == 1)
      begin
        always
          @(posedge READ_CLOCK_I or
            negedge RESET_READ_CLK_SYNC)
          if (!RESET_READ_CLK_SYNC)
            Oo <= 0;
          else
            if (lo > 8)
              Oo <= ~Oi;
            else
              Oo <= 0;
      end
    else
      if (g_LANE_WIDTH == 2)
        begin
          always
            @(posedge READ_CLOCK_I or
              negedge RESET_READ_CLK_SYNC)
            if (!RESET_READ_CLK_SYNC)
              begin
                Oo <= 0;
                oi <= 0;
              end
            else
              if (lo > 8)
                begin
                  oi <= oi + 1;
                  Oo <= ~oi;
                end
              else
                begin
                  Oo <= 0;
                  oi <= 0;
                end
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            always
              @(posedge READ_CLOCK_I or
                negedge RESET_READ_CLK_SYNC)
              if (!RESET_READ_CLK_SYNC)
                begin
                  Oo <= 0;
                  OOI <= 0;
                end
              else
                if (lo > 8)
                  begin
                    OOI <= OOI + 1;
                    Oo <= ~OOI[0] & ~OOI[1];
                  end
                else
                  begin
                    Oo <= 0;
                    OOI <= 0;
                  end
          end
  endgenerate
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    begin
      if (!RESET_READ_CLK_SYNC)
        io <= 0;
      else
        io <= { io[3 : 0], Oo };
    end
  assign Ii = (g_LANE_WIDTH == 1) ? io[0] : (g_LANE_WIDTH == 2) ? (io[0] | io[1]) : (g_LANE_WIDTH == 4) ? (io[0] | io[1] | io[2] | Oo) : io[0];
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    begin
      if (!RESET_READ_CLK_SYNC)
        line_valid_o <= 0;
      else
        line_valid_o <= (Ii) ? 1'b1 : 1'b0;
    end
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    begin
      if (!RESET_READ_CLK_SYNC)
        begin
          I0 <= 0;
          l0 <= 0;
        end
      else
        begin
          I0 <= o0;
          l0 <= I0;
        end
    end
  generate
    if (g_LANE_WIDTH == 1)
      begin
        always
          @(posedge READ_CLOCK_I or
            negedge RESET_READ_CLK_SYNC)
          if (!RESET_READ_CLK_SYNC)
            begin
              pixel_data_o <= 0;
            end
          else
            if (Ii)
              begin
                pixel_data_o <= I0;
              end
            else
              begin
                pixel_data_o <= 0;
              end
      end
    else
      if (g_LANE_WIDTH == 2)
        begin
          always
            @(posedge READ_CLOCK_I or
              negedge RESET_READ_CLK_SYNC)
            if (!RESET_READ_CLK_SYNC)
              begin
                li <= 0;
                pixel_data_o <= 0;
              end
            else
              if (Ii)
                begin
                  li <= li + 1;
                  pixel_data_o <= (li) ? I0[g_DATAWIDTH * 2 - 1 : g_DATAWIDTH] : I0[g_DATAWIDTH - 1 : 0];
                end
              else
                begin
                  li <= 0;
                  pixel_data_o <= 0;
                end
        end
      else
        if (g_LANE_WIDTH == 4)
          begin
            always
              @(posedge READ_CLOCK_I or
                negedge RESET_READ_CLK_SYNC)
              if (!RESET_READ_CLK_SYNC)
                begin
                  ii <= 0;
                  pixel_data_o <= 0;
                end
              else
                if (Ii)
                  begin
                    case (ii)
                      O:
                        begin
                          pixel_data_o <= l0[g_DATAWIDTH - 1 : 0];
                          ii <= I;
                        end
                      I:
                        begin
                          pixel_data_o <= l0[g_DATAWIDTH * 2 - 1 : g_DATAWIDTH];
                          ii <= l;
                        end
                      l:
                        begin
                          pixel_data_o <= l0[g_DATAWIDTH * 3 - 1 : g_DATAWIDTH * 2];
                          ii <= o;
                        end
                      o:
                        begin
                          pixel_data_o <= l0[g_DATAWIDTH * 4 - 1 : g_DATAWIDTH * 3];
                          ii <= O;
                        end
                      default:
                        pixel_data_o <= 0;
                    endcase
                  end
                else
                  begin
                    ii <= 0;
                    pixel_data_o <= 0;
                  end
          end
  endgenerate
  cdcfifo #(.ADDR_WIDTH(9), .DATA_WIDTH(ll)) IOI(.k_lim(9'h128),
                                                 .wrrstn(RESET_n_I),
                                                 .wrclk(CAM_CLOCK_I),
                                                 .wrreq(i1),
                                                 .wrdata(O0),
                                                 .wrfull(),
                                                 .wrusedw(),
                                                 .rdrstn(RESET_READ_CLK_SYNC),
                                                 .rdclk(READ_CLOCK_I),
                                                 .rdreq(Oo),
                                                 .rddata(o0),
                                                 .rdempty(Oi),
                                                 .rdusedw());
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    if (!RESET_READ_CLK_SYNC)
      ol <= 0;
    else
      ol <= Ii;
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_READ_CLK_SYNC)
    if (!RESET_READ_CLK_SYNC)
      begin
        il <= 0;
      end
    else
      begin
        il <= ol;
      end
  assign line_start_o = Ii & ~ol;
  assign line_end_o = ~ol & il;
endmodule
